@@ -1,6 +1,6 @@
 
 /*
- * Copyright 2014--2015 Jan Pazdziora
+ * Copyright 2014--2016 Jan Pazdziora
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -85,8 +85,7 @@ static const char * ap_escape_urlencoded(apr_pool_t * pool, const char * buffer)
 	while (*buffer) {
 		if (!apr_isalnum(*buffer) && !strchr(".-*_ ", *buffer)) {
 			*p++ = '%';
-			sprintf(p, "%02x", *p);
-			*p += 2;
+			p += snprintf(p, 3, "%02x", *buffer);
 		} else if (*buffer == ' ') {
 			*p++ = '+';
 		} else {
@@ -94,7 +93,7 @@ static const char * ap_escape_urlencoded(apr_pool_t * pool, const char * buffer)
 		}
 		buffer++;
 	}
-	*p++ = '\0';
+	*p = '\0';
 	return copy;
 }
 #endif
@@ -137,6 +136,12 @@ static const char * format_location(request_rec * r, const char * url, const cha
 
 module AP_MODULE_DECLARE_DATA authnz_pam_module;
 
+#ifdef APLOG_USE_MODULE
+#define SHOW_MODULE ""
+#else
+#define SHOW_MODULE "mod_authnz_pam: "
+#endif
+
 #define _REMOTE_USER_ENV_NAME "REMOTE_USER"
 #define _EXTERNAL_AUTH_ERROR_ENV_NAME "EXTERNAL_AUTH_ERROR"
 #define _PAM_STEP_AUTH 1
@@ -170,25 +175,27 @@ static authn_status pam_authenticate_with_login_password(request_rec * r, const
 			if (ret == PAM_NEW_AUTHTOK_REQD) {
 				authnz_pam_config_rec * conf = ap_get_module_config(r->per_dir_config, &authnz_pam_module);
 				if (conf && conf->expired_redirect_url) {
-					ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-						"mod_authnz_pam: PAM_NEW_AUTHTOK_REQD: redirect to [%s]",
+					ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+						SHOW_MODULE "PAM_NEW_AUTHTOK_REQD: redirect to [%s]",
 						conf->expired_redirect_url);
 					apr_table_addn(r->headers_out, "Location", format_location(r, conf->expired_redirect_url, login));
-					return HTTP_TEMPORARY_REDIRECT;
+					r->status = HTTP_TEMPORARY_REDIRECT;
+					ap_send_error_response(r, 0);
+					return AUTH_DENIED;
 				}
 			}
 		}
 	}
 	if (ret != PAM_SUCCESS) {
 		const char * strerr = pam_strerror(pamh, ret);
-		ap_log_error(APLOG_MARK, APLOG_WARNING, 0, r->server, "mod_authnz_pam: %s %s: %s", stage, param, strerr);
+		ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, r, SHOW_MODULE "%s %s: %s", stage, param, strerr);
 		apr_table_setn(r->subprocess_env, _EXTERNAL_AUTH_ERROR_ENV_NAME, apr_pstrdup(r->pool, strerr));
 		pam_end(pamh, ret);
 		return AUTH_DENIED;
 	}
 	apr_table_setn(r->subprocess_env, _REMOTE_USER_ENV_NAME, login);
 	r->user = apr_pstrdup(r->pool, login);
-	ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, r->server, "mod_authnz_pam: PAM authentication passed for user %s", login);
+	ap_log_rerror(APLOG_MARK, APLOG_INFO, 0, r, SHOW_MODULE "PAM authentication passed for user %s", login);
 	pam_end(pamh, ret);
 	return AUTH_GRANTED;
 }
@@ -270,7 +277,12 @@ static void register_hooks(apr_pool_t * p) {
 	APR_REGISTER_OPTIONAL_FN(pam_authenticate_with_login_password);
 }
 
-module AP_MODULE_DECLARE_DATA authnz_pam_module = {
+#ifdef AP_DECLARE_MODULE
+AP_DECLARE_MODULE(authnz_pam)
+#else
+module AP_MODULE_DECLARE_DATA authnz_pam_module
+#endif
+	= {
 	STANDARD20_MODULE_STUFF,
 	create_dir_conf,	/* Per-directory configuration handler */
 	NULL,			/* Merge handler for per-directory configurations */
